(function() {
    'use strict';

    angular.module('weChatApp')
        .factory('SentMessageService', ["$q", "Endpoints", "TerritoryService", "UserService", SentMessageService])
    ;

    function SentMessageService($q, endpoints, territoryService, userService) {
        var service = this;

        service.getSentMessageByUserId = function(userId, filterDates, includeAccountName, includeMultichannelContent, includeContentType) {
            var deferred = $q.defer();
            territoryService.getUserTerritories(userId).then(function (territoryResponse) {
                var territoryIds = _.map(territoryResponse.UserTerritory, "TerritoryId");
                userService.getUsersByTerritories(territoryIds).then(function (userResponse) {
                    var userIds = _.map(userResponse.UserTerritory, "UserId");
                    if (!!filterDates) {
                        endpoints.fetchSentMessageByUserIds(userIds, filterDates.startDatetime, filterDates.endDatetime).then(
                            sentMessageResponseProcessor(deferred, includeAccountName, includeMultichannelContent, includeContentType),
                            deferred.reject);
                    } else {
                        endpoints.fetchAllSentMessageByUserIds(userIds).then(
                            sentMessageResponseProcessor(deferred, includeAccountName, includeMultichannelContent, includeContentType),
                            deferred.reject);
                    }


                }, deferred.reject);
            }, deferred.reject);

            return deferred.promise;
        };

        function sentMessageResponseProcessor(deferred, includeAccountName, includeMultichannelContent, includeContentType) {
            return function(resp) {
                var sentMessageResp = resp.Sent_Message_vod__c;
                var subQueries = [];

                if(includeAccountName) {
                    var accountIds = _.map(_.uniqBy(sentMessageResp, "Account_vod__c"), "Account_vod__c");
                    var subDeferred1 = $q.defer();
                    endpoints.fetchAccountByIds(accountIds).then(subDeferred1.resolve, subDeferred1.reject);
                    subQueries.push({
                        name: "Account",
                        promise: subDeferred1.promise
                    });
                }

                if(includeMultichannelContent) {
                    var contentIds = _.map(_.uniqBy(sentMessageResp, "Content_vod__c"), "Content_vod__c");
                    var subDeferred2 = $q.defer();
                    endpoints.fetchMultichannelContentByIds(contentIds).then(subDeferred2.resolve, subDeferred2.reject);
                    subQueries.push({
                        name: "Multichannel_Content_vod__c",
                        promise: subDeferred2.promise
                    });
                }

                if(includeContentType) {
                    var contentTypeIds = _.map(_.uniqBy(sentMessageResp, "Content_Type_vod__c"), "Content_Type_vod__c");
                    var subDeferred3 = $q.defer();
                    endpoints.fetchContentTypeByIds(contentTypeIds).then(subDeferred3.resolve, subDeferred3.reject);
                    subQueries.push({
                        name: "Content_Type_vod__c",
                        promise: subDeferred3.promise
                    });
                }

                var userIds = _.map(sentMessageResp, "OwnerId");
                var subDeferred4 = $q.defer();
                userService.getUsersByIds(userIds).then(subDeferred4.resolve,subDeferred4.reject);
                subQueries.push({
                   name: "User",
                   promise: subDeferred4.promise
                });

                if(subQueries.length === 0) {
                    deferred.resolve(sentMessageResp);
                } else {
                    $q.all(_.map(subQueries, "promise")).then(function(subQueriesResp) {
                        subQueries.forEach(function(q, i) {
                            var subQueryType = q.name;
                            var accountIdToNameMap, mcContentIdToNameMap, contentTypeIdToNameMap, ownerIdToNameMap;

                            if(subQueryType === "Account") {
                                accountIdToNameMap = generateIdToNameMap(subQueriesResp[i].Account);
                            } else if(subQueryType === "Multichannel_Content_vod__c") {
                                mcContentIdToNameMap = generateIdToNameMap(subQueriesResp[i].Multichannel_Content_vod__c);
                            } else if(subQueryType === "Content_Type_vod__c") {
                                contentTypeIdToNameMap = generateIdToNameMap(subQueriesResp[i].Content_Type_vod__c);
                            } else if (subQueryType === "User") {
                                ownerIdToNameMap = generateIdToNameMap(subQueriesResp[i].User);
                            }

                            sentMessageResp.forEach(function(sm) {
                                if(!!sm.Account_vod__c && !!accountIdToNameMap) {
                                    sm.accountName = accountIdToNameMap[sm.Account_vod__c] || '';
                                }
                                if(!!sm.Content_vod__c && !!mcContentIdToNameMap) {
                                    sm.contentName = mcContentIdToNameMap[sm.Content_vod__c] || '';
                                }
                                if(!!sm.Content_Type_vod__c && !!contentTypeIdToNameMap) {
                                    sm.serviceAccountName = contentTypeIdToNameMap[sm.Content_Type_vod__c] || '';
                                }
                                if (!!sm.OwnerId && !!ownerIdToNameMap) {
                                    sm.ownerName = ownerIdToNameMap[sm.OwnerId] || '';
                                }
                            });
                        });
                        deferred.resolve(sentMessageResp);
                    }, function(error) {
                        console.warn(error);
                        deferred.resolve(sentMessageResp);
                    });
                }
            };
        }

        function generateIdToNameMap(objectList) {
            var map = {};
            objectList.forEach(function(obj) {
                map[obj.Id] = obj.Name;
            });
            return map;
        }

        return service;
    }
}());